from collections import OrderedDict

from flexmock import flexmock

from borgmatic.config import generate as module


def test_schema_to_sample_configuration_generates_config_with_examples():
    flexmock(module.yaml.comments).should_receive('CommentedMap').replace_with(OrderedDict)
    flexmock(module).should_receive('add_comments_to_configuration')
    schema = {
        'map': OrderedDict([
            (
                'section1', {
                    'map': {
                        'field1': OrderedDict([
                            ('example', 'Example 1')
                        ]),
                    },
                },
            ),
            (
                'section2', {
                    'map': OrderedDict([
                        ('field2', {'example': 'Example 2'}),
                        ('field3', {'example': 'Example 3'}),
                    ]),
                }
            ),
        ])
    }

    config = module._schema_to_sample_configuration(schema)

    assert config == OrderedDict([
        (
            'section1',
            OrderedDict([
                ('field1', 'Example 1'),
            ]),
        ),
        (
            'section2',
            OrderedDict([
                ('field2', 'Example 2'),
                ('field3', 'Example 3'),
            ]),
        )
    ])



from collections import defaultdict, OrderedDict, namedtuple
import os

from flexmock import flexmock
import pytest

from borgmatic.config import convert as module


Parsed_config = namedtuple('Parsed_config', ('location', 'storage', 'retention', 'consistency'))


def test_convert_section_generates_integer_value_for_integer_type_in_schema():
    flexmock(module.yaml.comments).should_receive('CommentedMap').replace_with(OrderedDict)
    source_section_config = OrderedDict([('check_last', '3')])
    section_schema = {'map': {'check_last': {'type': 'int'}}}

    destination_config = module._convert_section(source_section_config, section_schema)

    assert destination_config == OrderedDict([('check_last', 3)])


def test_convert_legacy_parsed_config_transforms_source_config_to_mapping():
    flexmock(module.yaml.comments).should_receive('CommentedMap').replace_with(OrderedDict)
    source_config = Parsed_config(
        location=OrderedDict([('source_directories', '/home'), ('repository', 'hostname.borg')]),
        storage=OrderedDict([('encryption_passphrase', 'supersecret')]),
        retention=OrderedDict([('keep_daily', 7)]),
        consistency=OrderedDict([('checks', 'repository')]),
    )
    source_excludes = ['/var']
    schema = {'map': defaultdict(lambda: {'map': {}})}

    destination_config = module.convert_legacy_parsed_config(source_config, source_excludes, schema)

    assert destination_config == OrderedDict([
        (
            'location',
            OrderedDict([
                ('source_directories', ['/home']),
                ('repositories', ['hostname.borg']),
                ('exclude_patterns', ['/var']),
            ]),
        ),
        ('storage', OrderedDict([('encryption_passphrase', 'supersecret')])),
        ('retention', OrderedDict([('keep_daily', 7)])),
        ('consistency', OrderedDict([('checks', ['repository'])])),
    ])


def test_convert_legacy_parsed_config_splits_space_separated_values():
    flexmock(module.yaml.comments).should_receive('CommentedMap').replace_with(OrderedDict)
    source_config = Parsed_config(
        location=OrderedDict([('source_directories', '/home /etc'), ('repository', 'hostname.borg')]),
        storage=OrderedDict(),
        retention=OrderedDict(),
        consistency=OrderedDict([('checks', 'repository archives')]),
    )
    source_excludes = ['/var']
    schema = {'map': defaultdict(lambda: {'map': {}})}

    destination_config = module.convert_legacy_parsed_config(source_config, source_excludes, schema) 

    assert destination_config == OrderedDict([
        (
            'location',
            OrderedDict([
                ('source_directories', ['/home', '/etc']),
                ('repositories', ['hostname.borg']),
                ('exclude_patterns', ['/var']),
            ]),
        ),
        ('storage', OrderedDict()),
        ('retention', OrderedDict()),
        ('consistency', OrderedDict([('checks', ['repository', 'archives'])])),
    ])


def test_guard_configuration_upgraded_raises_when_only_source_config_present():
    flexmock(os.path).should_receive('exists').with_args('config').and_return(True)
    flexmock(os.path).should_receive('exists').with_args('config.yaml').and_return(False)

    with pytest.raises(module.LegacyConfigurationNotUpgraded):
        module.guard_configuration_upgraded('config', 'config.yaml')


def test_guard_configuration_upgraded_does_not_raise_when_only_destination_config_present():
    flexmock(os.path).should_receive('exists').with_args('config').and_return(False)
    flexmock(os.path).should_receive('exists').with_args('config.yaml').and_return(True)

    module.guard_configuration_upgraded('config', 'config.yaml')


def test_guard_configuration_upgraded_does_not_raise_when_both_configs_present():
    flexmock(os.path).should_receive('exists').with_args('config').and_return(True)
    flexmock(os.path).should_receive('exists').with_args('config.yaml').and_return(True)

    module.guard_configuration_upgraded('config', 'config.yaml')


def test_guard_configuration_upgraded_does_not_raise_when_neither_config_present():
    flexmock(os.path).should_receive('exists').with_args('config').and_return(False)
    flexmock(os.path).should_receive('exists').with_args('config.yaml').and_return(False)

    module.guard_configuration_upgraded('config', 'config.yaml')


def test_guard_excludes_filename_omitted_raises_when_filename_provided():
    with pytest.raises(module.LegacyExcludesFilenamePresent):
        module.guard_excludes_filename_omitted(excludes_filename='/etc/borgmatic/excludes')


def test_guard_excludes_filename_omitted_does_not_raise_when_filename_not_provided():
    module.guard_excludes_filename_omitted(excludes_filename=None)



from borgmatic.commands import generate_config as module


def test_parse_arguments_with_no_arguments_uses_defaults():
    parser = module.parse_arguments()

    assert parser.destination_filename == module.DEFAULT_DESTINATION_CONFIG_FILENAME

def test_parse_arguments_with_filename_argument_overrides_defaults():
    parser = module.parse_arguments('--destination', 'config.yaml')

    assert parser.destination_filename == 'config.yaml'



from __future__ import print_function
from argparse import ArgumentParser
import os
from subprocess import CalledProcessError
import sys
import textwrap

from ruamel import yaml

from borgmatic import borg
from borgmatic.config import convert, generate, legacy, validate


DEFAULT_SOURCE_CONFIG_FILENAME = '/etc/borgmatic/config'
DEFAULT_SOURCE_EXCLUDES_FILENAME = '/etc/borgmatic/excludes'
DEFAULT_DESTINATION_CONFIG_FILENAME = '/etc/borgmatic/config.yaml'


def parse_arguments(*arguments):
    '''
    Given command-line arguments with which this script was invoked, parse the arguments and return
    them as an ArgumentParser instance.
    '''
    parser = ArgumentParser(
        description='''
            Convert legacy INI-style borgmatic configuration and excludes files to a single YAML
            configuration file. Note that this replaces any comments from the source files.
        '''
    )
    parser.add_argument(
        '-s', '--source-config',
        dest='source_config_filename',
        default=DEFAULT_SOURCE_CONFIG_FILENAME,
        help='Source INI-style configuration filename. Default: {}'.format(DEFAULT_SOURCE_CONFIG_FILENAME),
    )
    parser.add_argument(
        '-e', '--source-excludes',
        dest='source_excludes_filename',
        default=DEFAULT_SOURCE_EXCLUDES_FILENAME if os.path.exists(DEFAULT_SOURCE_EXCLUDES_FILENAME) else None,
        help='Excludes filename',
    )
    parser.add_argument(
        '-d', '--destination-config',
        dest='destination_config_filename',
        default=DEFAULT_DESTINATION_CONFIG_FILENAME,
        help='Destination YAML configuration filename. Default: {}'.format(DEFAULT_DESTINATION_CONFIG_FILENAME),
    )

    return parser.parse_args(arguments)


TEXT_WRAP_CHARACTERS = 80


def display_result(args):  # pragma: no cover
    result_lines = textwrap.wrap(
        'Your borgmatic configuration has been upgraded. Please review the result in {}.'.format(
            args.destination_config_filename
        ),
        TEXT_WRAP_CHARACTERS,
    )
    
    delete_lines = textwrap.wrap(
        'Once you are satisfied, you can safely delete {}{}.'.format(
            args.source_config_filename,
            ' and {}'.format(args.source_excludes_filename) if args.source_excludes_filename else '',
        ),
        TEXT_WRAP_CHARACTERS,
    )

    print('\n'.join(result_lines))
    print()
    print('\n'.join(delete_lines))


def main():  # pragma: no cover
    try:
        args = parse_arguments(*sys.argv[1:])
        schema = yaml.round_trip_load(open(validate.schema_filename()).read())
        source_config = legacy.parse_configuration(args.source_config_filename, legacy.CONFIG_FORMAT)
        source_config_file_mode = os.stat(args.source_config_filename).st_mode
        source_excludes = (
            open(args.source_excludes_filename).read().splitlines()
            if args.source_excludes_filename
            else []
        )

        destination_config = convert.convert_legacy_parsed_config(source_config, source_excludes, schema)

        generate.write_configuration(
            args.destination_config_filename,
            destination_config,
            mode=source_config_file_mode,
        )

        display_result(args)
    except (ValueError, OSError) as error:
        print(error, file=sys.stderr)
        sys.exit(1)



from __future__ import print_function
from argparse import ArgumentParser
import os
from subprocess import CalledProcessError
import sys

from ruamel import yaml

from borgmatic import borg
from borgmatic.config import convert, generate, validate


DEFAULT_DESTINATION_CONFIG_FILENAME = '/etc/borgmatic/config.yaml'


def parse_arguments(*arguments):
    '''
    Given command-line arguments with which this script was invoked, parse the arguments and return
    them as an ArgumentParser instance.
    '''
    parser = ArgumentParser(description='Generate a sample borgmatic YAML configuration file.')
    parser.add_argument(
        '-d', '--destination',
        dest='destination_filename',
        default=DEFAULT_DESTINATION_CONFIG_FILENAME,
        help='Destination YAML configuration filename. Default: {}'.format(DEFAULT_DESTINATION_CONFIG_FILENAME),
    )

    return parser.parse_args(arguments)


def main():  # pragma: no cover
    try:
        args = parse_arguments(*sys.argv[1:])

        generate.generate_sample_configuration(args.destination_filename, validate.schema_filename())

        print('Generated a sample configuration file at {}.'.format(args.destination_filename))
        print()
        print('Please edit the file to suit your needs. The values are just representative.')
        print('All fields are optional except where indicated.')
    except (ValueError, OSError) as error:
        print(error, file=sys.stderr)
        sys.exit(1)



from datetime import datetime
import glob
import itertools
import os
import platform
import re
import subprocess
import tempfile

from borgmatic.verbosity import VERBOSITY_SOME, VERBOSITY_LOTS


# Integration with Borg for actually handling backups.


COMMAND = 'borg'


def initialize(storage_config, command=COMMAND):
    passphrase = storage_config.get('encryption_passphrase')

    if passphrase:
        os.environ['{}_PASSPHRASE'.format(command.upper())] = passphrase


def _write_exclude_file(exclude_patterns=None):
    '''
    Given a sequence of exclude patterns, write them to a named temporary file and return it. Return
    None if no patterns are provided.
    '''
    if not exclude_patterns:
        return None

    exclude_file = tempfile.NamedTemporaryFile('w')
    exclude_file.write('\n'.join(exclude_patterns))
    exclude_file.flush()

    return exclude_file


def create_archive(
    verbosity, repository, location_config, storage_config, command=COMMAND,
):
    '''
    Given a vebosity flag, a storage config dict, a list of source directories, a local or remote
    repository path, a list of exclude patterns, and a command to run, create an attic archive.
    '''
    sources = tuple(
        itertools.chain.from_iterable(
            glob.glob(directory) or [directory]
            for directory in location_config['source_directories']
        )
    )

    exclude_file = _write_exclude_file(location_config.get('exclude_patterns'))
    exclude_flags = ('--exclude-from', exclude_file.name) if exclude_file else ()
    compression = storage_config.get('compression', None)
    compression_flags = ('--compression', compression) if compression else ()
    umask = storage_config.get('umask', None)
    umask_flags = ('--umask', str(umask)) if umask else ()
    one_file_system_flags = ('--one-file-system',) if location_config.get('one_file_system') else ()
    remote_path = location_config.get('remote_path')
    remote_path_flags = ('--remote-path', remote_path) if remote_path else ()
    verbosity_flags = {
        VERBOSITY_SOME: ('--info', '--stats',),
        VERBOSITY_LOTS: ('--debug', '--list', '--stats'),
    }.get(verbosity, ())

    full_command = (
        command, 'create',
        '{repo}::{hostname}-{timestamp}'.format(
            repo=repository,
            hostname=platform.node(),
            timestamp=datetime.now().isoformat(),
        ),
    ) + sources + exclude_flags + compression_flags + one_file_system_flags + \
        remote_path_flags + umask_flags + verbosity_flags

    subprocess.check_call(full_command)


def _make_prune_flags(retention_config):
    '''
    Given a retention config dict mapping from option name to value, tranform it into an iterable of
    command-line name-value flag pairs.

    For example, given a retention config of:

        {'keep_weekly': 4, 'keep_monthly': 6}

    This will be returned as an iterable of:

        (
            ('--keep-weekly', '4'),
            ('--keep-monthly', '6'),
        )
    '''
    return (
        ('--' + option_name.replace('_', '-'), str(retention_config[option_name]))
        for option_name, value in retention_config.items()
    )


def prune_archives(verbosity, repository, retention_config, command=COMMAND, remote_path=None):
    '''
    Given a verbosity flag, a local or remote repository path, a retention config dict, and a
    command to run, prune attic archives according the the retention policy specified in that
    configuration.
    '''
    remote_path_flags = ('--remote-path', remote_path) if remote_path else ()
    verbosity_flags = {
        VERBOSITY_SOME: ('--info', '--stats',),
        VERBOSITY_LOTS: ('--debug', '--stats'),
    }.get(verbosity, ())

    full_command = (
        command, 'prune',
        repository,
    ) + tuple(
        element
        for pair in _make_prune_flags(retention_config)
        for element in pair
    ) + remote_path_flags + verbosity_flags

    subprocess.check_call(full_command)


DEFAULT_CHECKS = ('repository', 'archives')


def _parse_checks(consistency_config):
    '''
    Given a consistency config with a "checks" list, transform it to a tuple of named checks to run.

    For example, given a retention config of:

        {'checks': ['repository', 'archives']}

    This will be returned as:

        ('repository', 'archives')

    If no "checks" option is present, return the DEFAULT_CHECKS. If the checks value is the string
    "disabled", return an empty tuple, meaning that no checks should be run.
    '''
    checks = consistency_config.get('checks', [])
    if checks == ['disabled']:
        return ()

    return tuple(check for check in checks if check.lower() not in ('disabled', '')) or DEFAULT_CHECKS


def _make_check_flags(checks, check_last=None):
    '''
    Given a parsed sequence of checks, transform it into tuple of command-line flags.

    For example, given parsed checks of:

        ('repository',)

    This will be returned as:

        ('--repository-only',)

    Additionally, if a check_last value is given, a "--last" flag will be added.
    '''
    last_flag = ('--last', str(check_last)) if check_last else ()
    if checks == DEFAULT_CHECKS:
        return last_flag

    return tuple(
        '--{}-only'.format(check) for check in checks
    ) + last_flag


def check_archives(verbosity, repository, consistency_config, command=COMMAND, remote_path=None):
    '''
    Given a verbosity flag, a local or remote repository path, a consistency config dict, and a
    command to run, check the contained attic archives for consistency.

    If there are no consistency checks to run, skip running them.
    '''
    checks = _parse_checks(consistency_config)
    check_last = consistency_config.get('check_last', None)
    if not checks:
        return

    remote_path_flags = ('--remote-path', remote_path) if remote_path else ()
    verbosity_flags = {
        VERBOSITY_SOME: ('--info',),
        VERBOSITY_LOTS: ('--debug',),
    }.get(verbosity, ())

    full_command = (
        command, 'check',
        repository,
    ) + _make_check_flags(checks, check_last) + remote_path_flags + verbosity_flags

    # The check command spews to stdout/stderr even without the verbose flag. Suppress it.
    stdout = None if verbosity_flags else open(os.devnull, 'w')

    subprocess.check_call(full_command, stdout=stdout, stderr=subprocess.STDOUT)



from collections import OrderedDict
from subprocess import STDOUT
import sys
import os

from flexmock import flexmock

from borgmatic import borg as module
from borgmatic.verbosity import VERBOSITY_SOME, VERBOSITY_LOTS


def test_initialize_with_passphrase_should_set_environment():
    orig_environ = os.environ

    try:
        os.environ = {}
        module.initialize({'encryption_passphrase': 'pass'}, command='borg')
        assert os.environ.get('BORG_PASSPHRASE') == 'pass'
    finally:
        os.environ = orig_environ


def test_initialize_without_passphrase_should_not_set_environment():
    orig_environ = os.environ

    try:
        os.environ = {}
        module.initialize({}, command='borg')
        assert os.environ.get('BORG_PASSPHRASE') == None
    finally:
        os.environ = orig_environ

def test_write_exclude_file_does_not_raise():
    temporary_file = flexmock(
        name='filename',
        write=lambda mode: None,
        flush=lambda: None,
    )
    flexmock(module.tempfile).should_receive('NamedTemporaryFile').and_return(temporary_file)

    module._write_exclude_file(['exclude'])


def test_write_exclude_file_with_empty_exclude_patterns_does_not_raise():
    module._write_exclude_file([])


def insert_subprocess_mock(check_call_command, **kwargs):
    subprocess = flexmock(STDOUT=STDOUT)
    subprocess.should_receive('check_call').with_args(check_call_command, **kwargs).once()
    flexmock(module).subprocess = subprocess


def insert_subprocess_never():
    subprocess = flexmock()
    subprocess.should_receive('check_call').never()
    flexmock(module).subprocess = subprocess


def insert_platform_mock():
    flexmock(module.platform).should_receive('node').and_return('host')


def insert_datetime_mock():
    flexmock(module).datetime = flexmock().should_receive('now').and_return(
        flexmock().should_receive('isoformat').and_return('now').mock
    ).mock


CREATE_COMMAND = ('borg', 'create', 'repo::host-now', 'foo', 'bar')


def test_create_archive_should_call_borg_with_parameters():
    flexmock(module).should_receive('_write_exclude_file')
    insert_subprocess_mock(CREATE_COMMAND)
    insert_platform_mock()
    insert_datetime_mock()

    module.create_archive(
        verbosity=None,
        repository='repo',
        location_config={
            'source_directories': ['foo', 'bar'],
            'repositories': ['repo'],
            'exclude_patterns': None,
        },
        storage_config={},
        command='borg',
    )


def test_create_archive_with_exclude_patterns_should_call_borg_with_excludes():
    flexmock(module).should_receive('_write_exclude_file').and_return(flexmock(name='excludes'))
    insert_subprocess_mock(CREATE_COMMAND + ('--exclude-from', 'excludes'))
    insert_platform_mock()
    insert_datetime_mock()

    module.create_archive(
        verbosity=None,
        repository='repo',
        location_config={
            'source_directories': ['foo', 'bar'],
            'repositories': ['repo'],
            'exclude_patterns': ['exclude'],
        },
        storage_config={},
        command='borg',
    )


def test_create_archive_with_verbosity_some_should_call_borg_with_info_parameter():
    flexmock(module).should_receive('_write_exclude_file')
    insert_subprocess_mock(CREATE_COMMAND + ('--info', '--stats',))
    insert_platform_mock()
    insert_datetime_mock()

    module.create_archive(
        verbosity=VERBOSITY_SOME,
        repository='repo',
        location_config={
            'source_directories': ['foo', 'bar'],
            'repositories': ['repo'],
            'exclude_patterns': None,
        },
        storage_config={},
        command='borg',
    )


def test_create_archive_with_verbosity_lots_should_call_borg_with_debug_parameter():
    flexmock(module).should_receive('_write_exclude_file')
    insert_subprocess_mock(CREATE_COMMAND + ('--debug', '--list', '--stats'))
    insert_platform_mock()
    insert_datetime_mock()

    module.create_archive(
        verbosity=VERBOSITY_LOTS,
        repository='repo',
        location_config={
            'source_directories': ['foo', 'bar'],
            'repositories': ['repo'],
            'exclude_patterns': None,
        },
        storage_config={},
        command='borg',
    )


def test_create_archive_with_compression_should_call_borg_with_compression_parameters():
    flexmock(module).should_receive('_write_exclude_file')
    insert_subprocess_mock(CREATE_COMMAND + ('--compression', 'rle'))
    insert_platform_mock()
    insert_datetime_mock()

    module.create_archive(
        verbosity=None,
        repository='repo',
        location_config={
            'source_directories': ['foo', 'bar'],
            'repositories': ['repo'],
            'exclude_patterns': None,
        },
        storage_config={'compression': 'rle'},
        command='borg',
    )


def test_create_archive_with_one_file_system_should_call_borg_with_one_file_system_parameters():
    flexmock(module).should_receive('_write_exclude_file')
    insert_subprocess_mock(CREATE_COMMAND + ('--one-file-system',))
    insert_platform_mock()
    insert_datetime_mock()

    module.create_archive(
        verbosity=None,
        repository='repo',
        location_config={
            'source_directories': ['foo', 'bar'],
            'repositories': ['repo'],
            'one_file_system': True,
            'exclude_patterns': None,
        },
        storage_config={},
        command='borg',
    )


def test_create_archive_with_remote_path_should_call_borg_with_remote_path_parameters():
    flexmock(module).should_receive('_write_exclude_file')
    insert_subprocess_mock(CREATE_COMMAND + ('--remote-path', 'borg1'))
    insert_platform_mock()
    insert_datetime_mock()

    module.create_archive(
        verbosity=None,
        repository='repo',
        location_config={
            'source_directories': ['foo', 'bar'],
            'repositories': ['repo'],
            'remote_path': 'borg1',
            'exclude_patterns': None,
        },
        storage_config={},
        command='borg',
    )


def test_create_archive_with_umask_should_call_borg_with_umask_parameters():
    flexmock(module).should_receive('_write_exclude_file')
    insert_subprocess_mock(CREATE_COMMAND + ('--umask', '740'))
    insert_platform_mock()
    insert_datetime_mock()

    module.create_archive(
        verbosity=None,
        repository='repo',
        location_config={
            'source_directories': ['foo', 'bar'],
            'repositories': ['repo'],
            'exclude_patterns': None,
        },
        storage_config={'umask': 740},
        command='borg',
    )


def test_create_archive_with_source_directories_glob_expands():
    flexmock(module).should_receive('_write_exclude_file')
    insert_subprocess_mock(('borg', 'create', 'repo::host-now', 'foo', 'food'))
    insert_platform_mock()
    insert_datetime_mock()
    flexmock(module.glob).should_receive('glob').with_args('foo*').and_return(['foo', 'food'])

    module.create_archive(
        verbosity=None,
        repository='repo',
        location_config={
            'source_directories': ['foo*'],
            'repositories': ['repo'],
            'exclude_patterns': None,
        },
        storage_config={},
        command='borg',
    )


def test_create_archive_with_non_matching_source_directories_glob_passes_through():
    flexmock(module).should_receive('_write_exclude_file')
    insert_subprocess_mock(('borg', 'create', 'repo::host-now', 'foo*'))
    insert_platform_mock()
    insert_datetime_mock()
    flexmock(module.glob).should_receive('glob').with_args('foo*').and_return([])

    module.create_archive(
        verbosity=None,
        repository='repo',
        location_config={
            'source_directories': ['foo*'],
            'repositories': ['repo'],
            'exclude_patterns': None,
        },
        storage_config={},
        command='borg',
    )


def test_create_archive_with_glob_should_call_borg_with_expanded_directories():
    flexmock(module).should_receive('_write_exclude_file')
    insert_subprocess_mock(('borg', 'create', 'repo::host-now', 'foo', 'food'))
    insert_platform_mock()
    insert_datetime_mock()
    flexmock(module.glob).should_receive('glob').with_args('foo*').and_return(['foo', 'food'])

    module.create_archive(
        verbosity=None,
        repository='repo',
        location_config={
            'source_directories': ['foo*'],
            'repositories': ['repo'],
            'exclude_patterns': None,
        },
        storage_config={},
        command='borg',
    )


BASE_PRUNE_FLAGS = (
    ('--keep-daily', '1'),
    ('--keep-weekly', '2'),
    ('--keep-monthly', '3'),
)


def test_make_prune_flags_should_return_flags_from_config():
    retention_config = OrderedDict(
        (
            ('keep_daily', 1),
            ('keep_weekly', 2),
            ('keep_monthly', 3),
        )
    )

    result = module._make_prune_flags(retention_config)

    assert tuple(result) == BASE_PRUNE_FLAGS


PRUNE_COMMAND = (
    'borg', 'prune', 'repo', '--keep-daily', '1', '--keep-weekly', '2', '--keep-monthly', '3',
)


def test_prune_archives_should_call_borg_with_parameters():
    retention_config = flexmock()
    flexmock(module).should_receive('_make_prune_flags').with_args(retention_config).and_return(
        BASE_PRUNE_FLAGS,
    )
    insert_subprocess_mock(PRUNE_COMMAND)

    module.prune_archives(
        verbosity=None,
        repository='repo',
        retention_config=retention_config,
        command='borg',
    )


def test_prune_archives_with_verbosity_some_should_call_borg_with_info_parameter():
    retention_config = flexmock()
    flexmock(module).should_receive('_make_prune_flags').with_args(retention_config).and_return(
        BASE_PRUNE_FLAGS,
    )
    insert_subprocess_mock(PRUNE_COMMAND + ('--info', '--stats',))

    module.prune_archives(
        repository='repo',
        verbosity=VERBOSITY_SOME,
        retention_config=retention_config,
        command='borg',
    )


def test_prune_archives_with_verbosity_lots_should_call_borg_with_debug_parameter():
    retention_config = flexmock()
    flexmock(module).should_receive('_make_prune_flags').with_args(retention_config).and_return(
        BASE_PRUNE_FLAGS,
    )
    insert_subprocess_mock(PRUNE_COMMAND + ('--debug', '--stats',))

    module.prune_archives(
        repository='repo',
        verbosity=VERBOSITY_LOTS,
        retention_config=retention_config,
        command='borg',
    )

def test_prune_archive_with_remote_path_should_call_borg_with_remote_path_parameters():
    retention_config = flexmock()
    flexmock(module).should_receive('_make_prune_flags').with_args(retention_config).and_return(
        BASE_PRUNE_FLAGS,
    )
    insert_subprocess_mock(PRUNE_COMMAND + ('--remote-path', 'borg1'))

    module.prune_archives(
        verbosity=None,
        repository='repo',
        retention_config=retention_config,
        command='borg',
        remote_path='borg1',
    )


def test_parse_checks_returns_them_as_tuple():
    checks = module._parse_checks({'checks': ['foo', 'disabled', 'bar']})

    assert checks == ('foo', 'bar')


def test_parse_checks_with_missing_value_returns_defaults():
    checks = module._parse_checks({})

    assert checks == module.DEFAULT_CHECKS


def test_parse_checks_with_blank_value_returns_defaults():
    checks = module._parse_checks({'checks': []})

    assert checks == module.DEFAULT_CHECKS


def test_parse_checks_with_disabled_returns_no_checks():
    checks = module._parse_checks({'checks': ['disabled']})

    assert checks == ()


def test_make_check_flags_with_checks_returns_flags():
    flags = module._make_check_flags(('foo', 'bar'))

    assert flags == ('--foo-only', '--bar-only')


def test_make_check_flags_with_default_checks_returns_no_flags():
    flags = module._make_check_flags(module.DEFAULT_CHECKS)

    assert flags == ()


def test_make_check_flags_with_checks_and_last_returns_flags_including_last():
    flags = module._make_check_flags(('foo', 'bar'), check_last=3)

    assert flags == ('--foo-only', '--bar-only', '--last', '3')


def test_make_check_flags_with_last_returns_last_flag():
    flags = module._make_check_flags(module.DEFAULT_CHECKS, check_last=3)

    assert flags == ('--last', '3')


def test_check_archives_should_call_borg_with_parameters():
    checks = flexmock()
    check_last = flexmock()
    consistency_config = flexmock().should_receive('get').and_return(check_last).mock
    flexmock(module).should_receive('_parse_checks').and_return(checks)
    flexmock(module).should_receive('_make_check_flags').with_args(checks, check_last).and_return(())
    stdout = flexmock()
    insert_subprocess_mock(
        ('borg', 'check', 'repo'),
        stdout=stdout, stderr=STDOUT,
    )
    insert_platform_mock()
    insert_datetime_mock()
    flexmock(sys.modules['builtins']).should_receive('open').and_return(stdout)
    flexmock(module.os).should_receive('devnull')

    module.check_archives(
        verbosity=None,
        repository='repo',
        consistency_config=consistency_config,
        command='borg',
    )


def test_check_archives_with_verbosity_some_should_call_borg_with_info_parameter():
    consistency_config = flexmock().should_receive('get').and_return(None).mock
    flexmock(module).should_receive('_parse_checks').and_return(flexmock())
    flexmock(module).should_receive('_make_check_flags').and_return(())
    insert_subprocess_mock(
        ('borg', 'check', 'repo', '--info'),
        stdout=None, stderr=STDOUT,
    )
    insert_platform_mock()
    insert_datetime_mock()

    module.check_archives(
        verbosity=VERBOSITY_SOME,
        repository='repo',
        consistency_config=consistency_config,
        command='borg',
    )


def test_check_archives_with_verbosity_lots_should_call_borg_with_debug_parameter():
    consistency_config = flexmock().should_receive('get').and_return(None).mock
    flexmock(module).should_receive('_parse_checks').and_return(flexmock())
    flexmock(module).should_receive('_make_check_flags').and_return(())
    insert_subprocess_mock(
        ('borg', 'check', 'repo', '--debug'),
        stdout=None, stderr=STDOUT,
    )
    insert_platform_mock()
    insert_datetime_mock()

    module.check_archives(
        verbosity=VERBOSITY_LOTS,
        repository='repo',
        consistency_config=consistency_config,
        command='borg',
    )


def test_check_archives_without_any_checks_should_bail():
    consistency_config = flexmock().should_receive('get').and_return(None).mock
    flexmock(module).should_receive('_parse_checks').and_return(())
    insert_subprocess_never()

    module.check_archives(
        verbosity=None,
        repository='repo',
        consistency_config=consistency_config,
        command='borg',
    )


def test_check_archives_with_remote_path_should_call_borg_with_remote_path_parameters():
    checks = flexmock()
    check_last = flexmock()
    consistency_config = flexmock().should_receive('get').and_return(check_last).mock
    flexmock(module).should_receive('_parse_checks').and_return(checks)
    flexmock(module).should_receive('_make_check_flags').with_args(checks, check_last).and_return(())
    stdout = flexmock()
    insert_subprocess_mock(
        ('borg', 'check', 'repo', '--remote-path', 'borg1'),
        stdout=stdout, stderr=STDOUT,
    )
    insert_platform_mock()
    insert_datetime_mock()
    flexmock(sys.modules['builtins']).should_receive('open').and_return(stdout)
    flexmock(module.os).should_receive('devnull')

    module.check_archives(
        verbosity=None,
        repository='repo',
        consistency_config=consistency_config,
        command='borg',
        remote_path='borg1',
    )



import subprocess


def test_setup_version_matches_news_version():
    setup_version = subprocess.check_output(('python', 'setup.py', '--version')).decode('ascii')
    news_version = open('NEWS').readline()

    assert setup_version == news_version



from __future__ import print_function
from argparse import ArgumentParser
import os
from subprocess import CalledProcessError
import sys

from borgmatic import borg
from borgmatic.config import convert, validate


LEGACY_CONFIG_FILENAME = '/etc/borgmatic/config'
DEFAULT_CONFIG_FILENAME = '/etc/borgmatic/config.yaml'
DEFAULT_EXCLUDES_FILENAME = '/etc/borgmatic/excludes'


def parse_arguments(*arguments):
    '''
    Given command-line arguments with which this script was invoked, parse the arguments and return
    them as an ArgumentParser instance.
    '''
    parser = ArgumentParser()
    parser.add_argument(
        '-c', '--config',
        dest='config_filename',
        default=DEFAULT_CONFIG_FILENAME,
        help='Configuration filename',
    )
    parser.add_argument(
        '--excludes',
        dest='excludes_filename',
        help='Excludes filename, deprecated in favor of exclude_patterns within configuration',
    )
    parser.add_argument(
        '-v', '--verbosity',
        type=int,
        help='Display verbose progress (1 for some, 2 for lots)',
    )

    return parser.parse_args(arguments)


def main():  # pragma: no cover
    try:
        args = parse_arguments(*sys.argv[1:])
        convert.guard_configuration_upgraded(LEGACY_CONFIG_FILENAME, args.config_filename)
        config = validate.parse_configuration(args.config_filename, validate.schema_filename())
        (location, storage, retention, consistency) = (
            config.get(section_name, {})
            for section_name in ('location', 'storage', 'retention', 'consistency')
        )
        remote_path = location.get('remote_path')

        borg.initialize(storage)

        for repository in location['repositories']:
            borg.prune_archives(args.verbosity, repository, retention, remote_path=remote_path)
            borg.create_archive(
                args.verbosity,
                repository,
                location,
                storage,
            )
            borg.check_archives(args.verbosity, repository, consistency, remote_path=remote_path)
    except (ValueError, OSError, CalledProcessError) as error:
        print(error, file=sys.stderr)
        sys.exit(1)



import os

from flexmock import flexmock
import pytest

from borgmatic.commands import convert_config as module


def test_parse_arguments_with_no_arguments_uses_defaults():
    flexmock(os.path).should_receive('exists').and_return(True)

    parser = module.parse_arguments()

    assert parser.source_config_filename == module.DEFAULT_SOURCE_CONFIG_FILENAME
    assert parser.source_excludes_filename == module.DEFAULT_SOURCE_EXCLUDES_FILENAME
    assert parser.destination_config_filename == module.DEFAULT_DESTINATION_CONFIG_FILENAME


def test_parse_arguments_with_filename_arguments_overrides_defaults():
    flexmock(os.path).should_receive('exists').and_return(True)

    parser = module.parse_arguments(
        '--source-config', 'config',
        '--source-excludes', 'excludes',
        '--destination-config', 'config.yaml',
    )

    assert parser.source_config_filename == 'config'
    assert parser.source_excludes_filename == 'excludes'
    assert parser.destination_config_filename == 'config.yaml'


def test_parse_arguments_with_missing_default_excludes_file_sets_filename_to_none():
    flexmock(os.path).should_receive('exists').and_return(False)

    parser = module.parse_arguments()

    assert parser.source_config_filename == module.DEFAULT_SOURCE_CONFIG_FILENAME
    assert parser.source_excludes_filename is None
    assert parser.destination_config_filename == module.DEFAULT_DESTINATION_CONFIG_FILENAME


def test_parse_arguments_with_invalid_arguments_exits():
    flexmock(os.path).should_receive('exists').and_return(True)

    with pytest.raises(SystemExit):
        module.parse_arguments('--posix-me-harder')



from io import StringIO
import os
import sys

from flexmock import flexmock
import pytest

from borgmatic.config import generate as module


def test_insert_newline_before_comment_does_not_raise():
    field_name = 'foo'
    config = module.yaml.comments.CommentedMap([(field_name, 33)])
    config.yaml_set_comment_before_after_key(key=field_name, before='Comment',)

    module._insert_newline_before_comment(config, field_name)


def test_write_configuration_does_not_raise():
    flexmock(os.path).should_receive('exists').and_return(False)
    flexmock(os).should_receive('makedirs')
    builtins = flexmock(sys.modules['builtins'])
    builtins.should_receive('open').and_return(StringIO())
    flexmock(os).should_receive('chmod')

    module.write_configuration('config.yaml', {})


def test_write_configuration_with_already_existing_file_raises():
    flexmock(os.path).should_receive('exists').and_return(True)

    with pytest.raises(FileExistsError):
        module.write_configuration('config.yaml', {})


def test_write_configuration_with_already_existing_directory_does_not_raise():
    flexmock(os.path).should_receive('exists').and_return(False)
    flexmock(os).should_receive('makedirs').and_raise(FileExistsError)
    builtins = flexmock(sys.modules['builtins'])
    builtins.should_receive('open').and_return(StringIO())
    flexmock(os).should_receive('chmod')

    module.write_configuration('config.yaml', {})


def test_add_comments_to_configuration_does_not_raise():
    # Ensure that it can deal with fields both in the schema and missing from the schema.
    config = module.yaml.comments.CommentedMap([('foo', 33), ('bar', 44), ('baz', 55)])
    schema = {
        'map': {
            'foo': {'desc': 'Foo'},
            'bar': {'desc': 'Bar'},
        }
    }

    module.add_comments_to_configuration(config, schema)


def test_generate_sample_configuration_does_not_raise():
    builtins = flexmock(sys.modules['builtins'])
    builtins.should_receive('open').with_args('schema.yaml').and_return('')
    flexmock(module).should_receive('write_configuration')
    flexmock(module).should_receive('_schema_to_sample_configuration')

    module.generate_sample_configuration('config.yaml', 'schema.yaml')



import os

from flexmock import flexmock
import pytest

from borgmatic.commands import borgmatic as module


def test_parse_arguments_with_no_arguments_uses_defaults():
    parser = module.parse_arguments()

    assert parser.config_filename == module.DEFAULT_CONFIG_FILENAME
    assert parser.excludes_filename == None
    assert parser.verbosity is None


def test_parse_arguments_with_filename_arguments_overrides_defaults():
    parser = module.parse_arguments('--config', 'myconfig', '--excludes', 'myexcludes')

    assert parser.config_filename == 'myconfig'
    assert parser.excludes_filename == 'myexcludes'
    assert parser.verbosity is None


def test_parse_arguments_with_verbosity_flag_overrides_default():
    parser = module.parse_arguments('--verbosity', '1')

    assert parser.config_filename == module.DEFAULT_CONFIG_FILENAME
    assert parser.excludes_filename == None
    assert parser.verbosity == 1


def test_parse_arguments_with_invalid_arguments_exits():
    with pytest.raises(SystemExit):
        module.parse_arguments('--posix-me-harder')



import os

from ruamel import yaml

from borgmatic.config import generate


def _convert_section(source_section_config, section_schema):
    '''
    Given a legacy Parsed_config instance for a single section, convert it to its corresponding
    yaml.comments.CommentedMap representation in preparation for actual serialization to YAML.

    Where integer types exist in the given section schema, convert their values to integers.
    '''
    destination_section_config = yaml.comments.CommentedMap([
        (
            option_name,
            int(option_value)
                if section_schema['map'].get(option_name, {}).get('type') == 'int' else option_value
        )
        for option_name, option_value in source_section_config.items()
    ])

    return destination_section_config


def convert_legacy_parsed_config(source_config, source_excludes, schema):
    '''
    Given a legacy Parsed_config instance loaded from an INI-style config file and a list of exclude
    patterns, convert them to a corresponding yaml.comments.CommentedMap representation in
    preparation for serialization to a single YAML config file.

    Additionally, use the given schema as a source of helpful comments to include within the
    returned CommentedMap.
    '''
    destination_config = yaml.comments.CommentedMap([
        (section_name, _convert_section(section_config, schema['map'][section_name]))
        for section_name, section_config in source_config._asdict().items()
    ])

    # Split space-seperated values into actual lists, make "repository" into a list, and merge in
    # excludes.
    location = destination_config['location']
    location['source_directories'] = source_config.location['source_directories'].split(' ')
    location['repositories'] = [location.pop('repository')]
    location['exclude_patterns'] = source_excludes

    if source_config.consistency.get('checks'):
        destination_config['consistency']['checks'] = source_config.consistency['checks'].split(' ')

    # Add comments to each section, and then add comments to the fields in each section.
    generate.add_comments_to_configuration(destination_config, schema)

    for section_name, section_config in destination_config.items():
        generate.add_comments_to_configuration(
            section_config,
            schema['map'][section_name],
            indent=generate.INDENT,
        )

    return destination_config


class LegacyConfigurationNotUpgraded(FileNotFoundError):
    def __init__(self):
        super(LegacyConfigurationNotUpgraded, self).__init__(
            '''borgmatic changed its configuration file format in version 1.1.0 from INI-style
to YAML. This better supports validation, and has a more natural way to express
lists of values. To upgrade your existing configuration, run:

    sudo upgrade-borgmatic-config

That will generate a new YAML configuration file at /etc/borgmatic/config.yaml
(by default) using the values from both your existing configuration and excludes
files. The new version of borgmatic will consume the YAML configuration file
instead of the old one.'''
        )


def guard_configuration_upgraded(source_config_filename, destination_config_filename):
    '''
    If legacy souce configuration exists but destination upgraded config doesn't, raise
    LegacyConfigurationNotUpgraded.

    The idea is that we want to alert the user about upgrading their config if they haven't already.
    '''
    if os.path.exists(source_config_filename) and not os.path.exists(destination_config_filename):
        raise LegacyConfigurationNotUpgraded()


class LegacyExcludesFilenamePresent(FileNotFoundError):
    def __init__(self):
        super(LegacyExcludesFilenamePresent, self).__init__(
            '''borgmatic changed its configuration file format in version 1.1.0 from INI-style
to YAML. This better supports validation, and has a more natural way to express
lists of values. The new configuration file incorporates excludes, so you no
longer need to provide an excludes filename on the command-line with an
"--excludes" argument.

Please remove the "--excludes" argument and run borgmatic again.'''
        )


def guard_excludes_filename_omitted(excludes_filename):
    if excludes_filename != None:
        raise LegacyExcludesFilenamePresent()



import logging
import sys
import warnings

import pkg_resources
import pykwalify.core
import pykwalify.errors
from ruamel import yaml


def schema_filename():
    '''
    Path to the installed YAML configuration schema file, used to validate and parse the
    configuration.
    '''
    return pkg_resources.resource_filename('borgmatic', 'config/schema.yaml')


class Validation_error(ValueError):
    '''
    A collection of error message strings generated when attempting to validate a particular
    configurartion file.
    '''
    def __init__(self, config_filename, error_messages):
        self.config_filename = config_filename
        self.error_messages = error_messages


def parse_configuration(config_filename, schema_filename):
    '''
    Given the path to a config filename in YAML format and the path to a schema filename in
    pykwalify YAML schema format, return the parsed configuration as a data structure of nested
    dicts and lists corresponding to the schema. Example return value:

       {'location': {'source_directories': ['/home', '/etc'], 'repository': 'hostname.borg'},
       'retention': {'keep_daily': 7}, 'consistency': {'checks': ['repository', 'archives']}}

    Raise FileNotFoundError if the file does not exist, PermissionError if the user does not
    have permissions to read the file, or Validation_error if the config does not match the schema.
    '''
    try:
        config = yaml.round_trip_load(open(config_filename))
        schema = yaml.round_trip_load(open(schema_filename))
    except yaml.error.YAMLError as error:
        raise Validation_error(config_filename, (str(error),))

    # pykwalify gets angry if the example field is not a string. So rather than bend to its will,
    # simply remove all examples before passing the schema to pykwalify.
    for section_name, section_schema in schema['map'].items():
        for field_name, field_schema in section_schema['map'].items():
            field_schema.pop('example')

    validator = pykwalify.core.Core(source_data=config, schema_data=schema)
    parsed_result = validator.validate(raise_exception=False)

    if validator.validation_errors:
        raise Validation_error(config_filename, validator.validation_errors)

    return parsed_result


def display_validation_error(validation_error):
    '''
    Given a Validation_error, display its error messages to stderr.
    '''
    print(
        'An error occurred while parsing a configuration file at {}:'.format(
            validation_error.config_filename
        ),
        file=sys.stderr,
    )

    for error in validation_error.error_messages:
        print(error, file=sys.stderr)



from setuptools import setup, find_packages


VERSION = '1.1.2'


setup(
    name='borgmatic',
    version=VERSION,
    description='A wrapper script for Borg backup software that creates and prunes backups',
    author='Dan Helfman',
    author_email='witten@torsion.org',
    url='https://torsion.org/borgmatic',
    download_url='https://torsion.org/hg/borgmatic/archive/%s.tar.gz' % VERSION,
    classifiers=(
        'Development Status :: 5 - Production/Stable',
        'Environment :: Console',
        'Intended Audience :: System Administrators',
        'License :: OSI Approved :: GNU General Public License v3 (GPLv3)',
        'Programming Language :: Python',
        'Topic :: Security :: Cryptography',
        'Topic :: System :: Archiving :: Backup',
    ),
    packages=find_packages(),
    entry_points={
        'console_scripts': [
            'borgmatic = borgmatic.commands.borgmatic:main',
            'upgrade-borgmatic-config = borgmatic.commands.convert_config:main',
            'generate-borgmatic-config = borgmatic.commands.generate_config:main',
        ]
    },
    obsoletes=[
        'atticmatic',
    ],
    install_requires=(
        'pykwalify',
        'ruamel.yaml<=0.15',
        'setuptools',
    ),
    tests_require=(
        'flexmock',
        'pytest',
    ),
    include_package_data=True,
)



from collections import OrderedDict
import os

from ruamel import yaml


INDENT = 4


def _insert_newline_before_comment(config, field_name):
    '''
    Using some ruamel.yaml black magic, insert a blank line in the config right befor the given
    field and its comments.
    '''
    config.ca.items[field_name][1].insert(
        0,
        yaml.tokens.CommentToken('\n', yaml.error.CommentMark(0), None),
    )


def _schema_to_sample_configuration(schema, level=0):
    '''
    Given a loaded configuration schema, generate and return sample config for it. Include comments
    for each section based on the schema "desc" description.
    '''
    example = schema.get('example')
    if example:
        return example

    config = yaml.comments.CommentedMap([
        (
            section_name,
            _schema_to_sample_configuration(section_schema, level + 1),
        )
        for section_name, section_schema in schema['map'].items()
    ])

    add_comments_to_configuration(config, schema, indent=(level * INDENT))

    return config


def write_configuration(config_filename, config, mode=0o600):
    '''
    Given a target config filename and a config data structure of nested OrderedDicts, write out the
    config to file as YAML. Create any containing directories as needed.
    '''
    if os.path.exists(config_filename):
        raise FileExistsError('{} already exists. Aborting.'.format(config_filename))

    try:
        os.makedirs(os.path.dirname(config_filename), mode=0o700)
    except (FileExistsError, FileNotFoundError):
        pass

    with open(config_filename, 'w') as config_file:
        config_file.write(yaml.round_trip_dump(config, indent=INDENT, block_seq_indent=INDENT))

    os.chmod(config_filename, mode)


def add_comments_to_configuration(config, schema, indent=0):
    '''
    Using descriptions from a schema as a source, add those descriptions as comments to the given
    config before each field. This function only adds comments for the top-most config map level.
    Indent the comment the given number of characters.
    '''
    for index, field_name in enumerate(config.keys()):
        field_schema = schema['map'].get(field_name, {})
        description = field_schema.get('desc')

        # No description to use? Skip it.
        if not field_schema or not description:
            continue

        config.yaml_set_comment_before_after_key(
            key=field_name,
            before=description,
            indent=indent,
        )
        if index > 0:
            _insert_newline_before_comment(config, field_name)


def generate_sample_configuration(config_filename, schema_filename):
    '''
    Given a target config filename and the path to a schema filename in pykwalify YAML schema
    format, write out a sample configuration file based on that schema.
    '''
    schema = yaml.round_trip_load(open(schema_filename))
    config = _schema_to_sample_configuration(schema)

    write_configuration(config_filename, config)



from io import StringIO

from collections import OrderedDict
import string

from borgmatic.config import legacy as module


def test_parse_section_options_with_punctuation_should_return_section_options():
    parser = module.RawConfigParser()
    parser.readfp(StringIO('[section]\nfoo: {}\n'.format(string.punctuation)))

    section_format = module.Section_format(
        'section',
        (module.Config_option('foo', str, required=True),),
    )

    config = module.parse_section_options(parser, section_format)

    assert config == OrderedDict(
        (
            ('foo', string.punctuation),
        )
    )



import io
import string
import sys
import os

from flexmock import flexmock
import pytest

from borgmatic.config import validate as module


def test_schema_filename_returns_plausable_path():
    schema_path = module.schema_filename()    

    assert schema_path.endswith('/schema.yaml')


def mock_config_and_schema(config_yaml):
    '''
    Set up mocks for the config config YAML string and the default schema so that the code under
    test consumes them when parsing the configuration.
    '''
    config_stream = io.StringIO(config_yaml)
    schema_stream = open(module.schema_filename())
    builtins = flexmock(sys.modules['builtins'])
    builtins.should_receive('open').with_args('config.yaml').and_return(config_stream)
    builtins.should_receive('open').with_args('schema.yaml').and_return(schema_stream)


def test_parse_configuration_transforms_file_into_mapping():
    mock_config_and_schema(
        '''
        location:
            source_directories:
                - /home
                - /etc

            repositories:
                - hostname.borg

        retention:
            keep_daily: 7

        consistency:
            checks:
                - repository
                - archives
        '''
    )

    result = module.parse_configuration('config.yaml', 'schema.yaml')

    assert result == {
        'location': {'source_directories': ['/home', '/etc'], 'repositories': ['hostname.borg']},
        'retention': {'keep_daily': 7},
        'consistency': {'checks': ['repository', 'archives']},
    }


def test_parse_configuration_passes_through_quoted_punctuation():
    escaped_punctuation = string.punctuation.replace('\\', r'\\').replace('"', r'\"')

    mock_config_and_schema(
        '''
        location:
            source_directories:
                - /home

            repositories:
                - "{}.borg"
        '''.format(escaped_punctuation)
    )

    result = module.parse_configuration('config.yaml', 'schema.yaml')

    assert result == {
        'location': {
            'source_directories': ['/home'],
            'repositories': ['{}.borg'.format(string.punctuation)],
        },
    }


def test_parse_configuration_raises_for_missing_config_file():
    with pytest.raises(FileNotFoundError):
        module.parse_configuration('config.yaml', 'schema.yaml')


def test_parse_configuration_raises_for_missing_schema_file():
    mock_config_and_schema('')
    builtins = flexmock(sys.modules['builtins'])
    builtins.should_receive('open').with_args('schema.yaml').and_raise(FileNotFoundError)

    with pytest.raises(FileNotFoundError):
        module.parse_configuration('config.yaml', 'schema.yaml')


def test_parse_configuration_raises_for_syntax_error():
    mock_config_and_schema('foo:\nbar')

    with pytest.raises(ValueError):
        module.parse_configuration('config.yaml', 'schema.yaml')


def test_parse_configuration_raises_for_validation_error():
    mock_config_and_schema(
        '''
        location:
            source_directories: yes
            repositories:
                - hostname.borg
        '''
    )

    with pytest.raises(module.Validation_error):
        module.parse_configuration('config.yaml', 'schema.yaml')


def test_display_validation_error_does_not_raise():
    flexmock(sys.modules['builtins']).should_receive('print')
    error = module.Validation_error('config.yaml', ('oops', 'uh oh'))

    module.display_validation_error(error)
